class Ilum < Formula
  include Language::Python::Virtualenv

  desc "CLI installer and management tool for the Ilum Data Lakehouse platform"
  homepage "https://ilum.cloud"
  url "https://files.pythonhosted.org/packages/source/i/ilum/ilum-0.1.0.tar.gz"
  sha256 "PLACEHOLDER_SHA256"
  license "Apache-2.0"

  depends_on "python@3.12"

  # Resource blocks will be generated by CI after each release.
  # Run: brew update-python-resources ilum

  def install
    virtualenv_install_with_resources
  end

  test do
    assert_match "ilum", shell_output("#{bin}/ilum --version")
  end
end
